//
//  FlatGLWidget.h
//  ParchmentFlattener
//
//  Copyright (c) 2014 Kazim Pal.
//  Released under the MIT License.
//
#pragma once
#include <QGLWidget>
#include <QVector3D>
#include <igl/trackball.h>
#include <Eigen/Dense>

class IGLMesh;
class ScaleConstraint;
class AutomationEngine;

using namespace Eigen;

class FlatGLWidget : public QGLWidget
{
Q_OBJECT
public:
	FlatGLWidget(void);
    FlatGLWidget(QWidget *parent = 0, QGLWidget *share = 0);
    FlatGLWidget(IGLMesh *_mesh, QWidget *parent = 0, QGLWidget *share = 0);
	~FlatGLWidget(void);

    IGLMesh *mesh;

    GLuint *meshTextures;
    int numTextures;
    void loadTextures();
    bool texturesLoaded;

    void resetRotation();

    bool renderOriginal;
    bool renderConstraints;
    bool renderScaleField;
    bool renderActualMesh;
    bool renderNormals;
    bool renderInterpolation;
    bool renderEnhanced;
    bool renderStretch;

    int window_width;
    int window_height;

    void renderOriginalMesh();

    QImage renderScreenToQImage(int w, int h);
    QImage renderTargetToQImage(Vector3d render_target, int w, int h);

    void generateSuggestionTail(int numSteps = -1);
    void generateSuggestionTailDynamic();
    std::vector<QPoint> smoothBaseline(std::vector<QPoint> ps1, int n, int from, int to);
    int selectSuggestionTailPoint(int mouseX, int mouseY);

    void startInterpolation();
    void renderInterpolatedMesh();
    double camz_interp;

    double camz;
    float znear,zfar;
    bool smoothZoom;

    Vector3d getTarget();
    void setTarget(Vector3d _target);

    void removeBlackBorder();
    void removeBlackBorderBySize();

signals:
    void hoverChangedSignal();
    void hoverChanged(int X, int Y);

public slots:
    void smoothingIterationWasDone();
    void flatteningIterationWasDone();
    void interpolationStep();
    void zoomIn();
    void zoomOut();

protected:
    virtual void initializeGL();
    virtual void paintGL();
    virtual void resizeGL(int width, int height);
    virtual void mousePressEvent(QMouseEvent * e);
    virtual void mouseMoveEvent(QMouseEvent * e);
    virtual void mouseReleaseEvent(QMouseEvent * e);
    virtual void mouseClickEvent(QMouseEvent *e);
    virtual void wheelEvent(QWheelEvent * e);
    void renderMesh();
    void renderLineConstraints();
    void renderLineConstraintsW();
    void renderScaleConstraints();
    void renderScaleConstraintsW();
    void renderScaleConstraint(ScaleConstraint sc);
    bool clickOnMesh(double winX, double winY, int *k, double *s, double *t, Vector3d *p);
    void handleClick(int winX, int winY);
    void handleClickRight(int winX, int winY);
    Vector3d getSurfacePoint(int k, double s, double t);
    int selectLineConstraintKeypoint(int mouseX, int mouseY);
    void setProjectionMatrixGL();
    void setModelViewMatrixGL();
    void setModelViewMatrixGL(Vector3d mvTarget);
    void setHoverPoint(int winX, int winY);

private:
    QColor qtBlack;
    void init();
    double rot_angle_old;
    double rot_angle_new;
    AutomationEngine *automationEngine;

    Vector3d target;
    Vector3d target_new;
    Vector3d currentGravity;
    Vector3d currentUpVec;
    Vector3d currentRightVec;

    double *quat_rot_new;
    double *quat_rot_current;
    double *rotation_matrix;

    double interpolation_t;
    QTimer *interpTimer;

    QPoint m_lastPoint;
    bool m_mouseClick;

    QPointF fullProjectionGL(double x, double y, double z);
    Vector3d fullUnProjectionGL(double winx, double winy, double winz);
};

